/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include "outlook/OutlookConfig.h"
#include "spdm/common/DeviceManager.h"
#include "spdm/common/DeviceManagerFactory.h"
#include "spds/common/Config.h"
#include "outlook/winmaincpp.h"


OutlookConfig::OutlookConfig() {
}

/*
* Stores the informations about name and id folder of outlook application. It writes
* pair key - value like contacts - 00000000609FE2FF0477CB429A576B081F5BB9BD42810000...
* actually not used..
*/

bool OutlookConfig::saveOutlookConfigToDM(wchar_t* propertyName, wchar_t* propertyValue, wchar_t* context_spds) { 
    
    wchar_t context[DIM_MANAGEMENT_PATH];
    int nc = 0;
    bool ret = false;

    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();

    wsprintf(context, TEXT("%s/%s"), APPLICATION_URI, context_spds);    

    ManagementNode* spdsConfig = dm->getManagementNode(context);
    if (spdsConfig == NULL) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("SyncManager configuration not found: %s"), context);
        goto finally;
    }

    spdsConfig->setPropertyValue(propertyName, propertyValue);   
    
    ret = true;

finally:

    if (dm != NULL) {
        delete dm;
    }

    if (spdsConfig != NULL) {
        delete spdsConfig;
    }

    return ret;
}

int OutlookConfig::setConfigParameters (wchar_t* syncURL, wchar_t* serverName, wchar_t* username, 
                                        wchar_t* password, wchar_t* deviceId,  wchar_t* errorMsg) {
    
    wchar_t context[DIM_MANAGEMENT_PATH];
    int nc = 0;
    int ret = 0;

    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();

    wsprintf(context, TEXT("%s/%s"), APPLICATION_URI, CONTEXT_SPDS_SYNCML);

    ManagementNode* spdsConfig = dm->getManagementNode(context);
    if (spdsConfig == NULL) {
        wsprintf(errorMsg, TEXT("SyncManager configuration not found: %s"), context);
                ret = -1;
        goto finally;
    }
        
    spdsConfig->setPropertyValue(PROPERTY_SYNC_URL, syncURL);    
    spdsConfig->setPropertyValue(PROPERTY_SERVER_NAME, serverName);
    spdsConfig->setPropertyValue(PROPERTY_USERNAME, username);
    spdsConfig->setPropertyValue(PROPERTY_PASSWORD, password);
    spdsConfig->setPropertyValue(PROPERTY_DEVICE_ID, deviceId);     
        
        
    wsprintf(context, TEXT("%s/%s"), APPLICATION_URI, CONTEXT_SPDS_SOURCES);
    spdsConfig = dm->getManagementNode(context);

    if (spdsConfig == NULL) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("SyncManager configuration not found: %s"), context);
        goto finally;
    }
        // if nc <= 0 create source/contacts and sources/appointments
    nc = spdsConfig->getChildrenCount();
    init (nc);


    ret = 0;

finally:

    if (dm != NULL) {
        delete dm;
    }

    if (spdsConfig != NULL) {
        delete spdsConfig;
    }

    return ret;
}


int OutlookConfig::getConfigParameter (wchar_t* name, wchar_t* value) {

    wchar_t rootContext[DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, APPLICATION_URI);
    wchar_t tmp[512];
    wsprintf(tmp, TEXT("empty"));

    Config* config = NULL;  
        
    config = new Config(rootContext);
        BOOL ret = config->readFromDM();

    if (wcscmp(name, SPDM_SYNCURL) == 0) {
        wsprintf(tmp, TEXT("%s"),  config->getSyncURL());               
    }
    else if (wcscmp(name, SPDM_SERVERNAME) == 0) {
        wsprintf(tmp, TEXT("%s"),  config->getServerName());
    }
    else if (wcscmp(name, SPDM_PASSWORD) == 0) {
        wsprintf(tmp, TEXT("%s"),  config->getPassword());
    }
    else if (wcscmp(name, SPDM_USERNAME) == 0) {
        wsprintf(tmp, TEXT("%s"),  config->getUsername());
    }
    else if (wcscmp(name, SPDM_DEVICEID) == 0) {
        wsprintf(tmp, TEXT("%s"),  config->getDeviceId());
    }
        
    wcscpy(value, tmp);     
    
    if (config != NULL)
        delete config;
    
    return 0;

}


int OutlookConfig::getSourceParameter (wchar_t* name, wchar_t* propertyName, wchar_t* value) {

    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, APPLICATION_URI);

    Config* config = NULL;

    config = new Config(rootContext);
    config->readFromDM();
    wchar_t tmp[50];
            
    int sourceCount = 0;
    sourceCount = config->getSyncSourceCount();
    
    if (sourceCount < 0) {
        init(sourceCount);              
    }

    SyncSourceConfig syncSourceConfig = SyncSourceConfig();

    config->readFromDM();
    sourceCount = config->getSyncSourceCount();

    
    if (wcscmp(name, SYNC_APPOINTMENTS) == 0) {                             
        config->getSyncSourceConfig(APPOINTMENTS_NAME, syncSourceConfig);                                       
    }
    else if (wcscmp(name, SYNC_CONTACTS) == 0) {
    
        config->getSyncSourceConfig(CONTACTS_NAME, syncSourceConfig);   
    }
    else if (wcscmp(name, SYNC_TASKS) == 0) {

        config->getSyncSourceConfig(TASKS_NAME, syncSourceConfig);       
    }
    else if (wcscmp(name, SYNC_NOTES) == 0) {
        config->getSyncSourceConfig(NOTES_NAME, syncSourceConfig);              
    }
    
    if (wcscmp(propertyName, PROPERTY_SOURCE_SYNC) == 0) {
        
        wsprintf(tmp, TEXT("%s"),  syncSourceConfig.getSync(NULL));
        
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_URI) == 0) {
        
        wsprintf(tmp, TEXT("%s"),  syncSourceConfig.getURI(NULL));                      
        
    }
        
    // overwrite the terminator char \0 of wchar_t* array. It doesn't work fine in VB!!
    wcscpy(value, tmp);     
    //value[wcslen(value)] = L' ';

    if (config != NULL)
        delete config;
    return 0;

}


int OutlookConfig::setSourceParameter (wchar_t* name, wchar_t* propertyName, wchar_t* value, wchar_t* errorMsg) {
        
    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, APPLICATION_URI);
    
    Config* config = NULL;

    config = new Config(rootContext);
    int sourceCount = 0;
    sourceCount = config->getSyncSourceCount();
    init(sourceCount);

    SyncSourceConfig syncSourceConfig = SyncSourceConfig();

    if (wcscmp(name, SYNC_APPOINTMENTS) == 0) {             
        
        config->getSyncSourceConfig(APPOINTMENTS_NAME, syncSourceConfig);               
    }
    else if (wcscmp(name, SYNC_CONTACTS) == 0) {    
        
        config->getSyncSourceConfig(CONTACTS_NAME, syncSourceConfig);           
    }
    else if (wcscmp(name, SYNC_TASKS) == 0) {   
            
        config->getSyncSourceConfig(TASKS_NAME, syncSourceConfig);              
    }
    else if (wcscmp(name, SYNC_NOTES) == 0) {   
            
        config->getSyncSourceConfig(NOTES_NAME, syncSourceConfig);                      
    }

    if (wcscmp(propertyName, PROPERTY_SOURCE_SYNC) == 0)
        syncSourceConfig.setSync(value);
    else if (wcscmp(propertyName, PROPERTY_SOURCE_URI) == 0)
        syncSourceConfig.setURI(value);
    
    config->setSyncSourceConfig(syncSourceConfig);
    config->saveToDM();
    
    //always true
    wsprintf(errorMsg, TEXT("success"));    

    if (config != NULL)
        delete config;
    return 0;

}

/*
* method for init sources...
*/

int  OutlookConfig::init (int sourceCount) {


    if (sourceCount <= 0) {

        saveOutlookConfigToDM(TEXT("name"), CONTACTS_NAME, CONTEXT_SPDS_SOURCES_CONTACTS );
        saveOutlookConfigToDM(TEXT("name"), APPOINTMENTS_NAME, CONTEXT_SPDS_SOURCES_APPOINTMENTS );
        saveOutlookConfigToDM(TEXT("sync"), TEXT("none"), CONTEXT_SPDS_SOURCES_CONTACTS );
        saveOutlookConfigToDM(TEXT("sync"), TEXT("none"), CONTEXT_SPDS_SOURCES_APPOINTMENTS );
    }
    return 0;

}

int OutlookConfig::config() {
        
    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, APPLICATION_URI);
    
    Config* config = NULL;

    config = new Config(rootContext);
    int sourceCount = 0;
    sourceCount = config->getSyncSourceCount();
    init(sourceCount);

    if (config != NULL)
        delete config;
    
    return 0;
}


int OutlookConfig::setClientConfigurationOutlook(wchar_t* sourceName, wchar_t* propertyName, 
                                                  wchar_t* propertyValue) { 
    
    wchar_t context[DIM_MANAGEMENT_PATH];
    int nc = 0;
    int ret = 0;

    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();
    
    wsprintf(context, APPLICATION_URI);
    wsprintf(context, TEXT("%s/%s"), context, CONTEXT_SPDS_SOURCESPROP);

    ManagementNode* spdsConfig = dm->getManagementNode(context);
    if (spdsConfig == NULL) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("SyncManager configuration not found: %s"), context);
        goto finally;
    }
    if (wcscmp(sourceName, TEXT("")) == 0) {
        
        
        if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_SYSTEMTIME_TO_LOCALTIME) == 0)
            spdsConfig->setPropertyValue(PROPERTY_SOURCEPROP_SYSTEMTIME_TO_LOCALTIME, propertyValue);         
        else
            spdsConfig->setPropertyValue(propertyName, propertyValue); 
       
    }


finally:

    if (dm != NULL) {
        delete dm;
    }

    if (spdsConfig != NULL) {
        delete spdsConfig;
    }

    return ret;
}
/*
* Method to get the client configurations. These property are set by installing process.
* These are extra parameters not used in synchronization process 
* but are needed for example to show or hide some fields in GUI interface.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

int OutlookConfig::getClientConfigurationOutlook (wchar_t* sourceName, wchar_t* propertyName, 
                                                  wchar_t* propertyValue) {    

    int nc = 0;
    wchar_t context[DIM_MANAGEMENT_PATH];
    wchar_t s [DIM_MANAGEMENT_PATH];
        
    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, APPLICATION_URI);

    wchar_t buf[VALUESIZE]; 

    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();     
    
    wsprintf(context, TEXT("%s/%s"), rootContext, CONTEXT_SPDS_SOURCESPROP);
    
    ManagementNode* spdsConfig = dm->getManagementNode(context);
    
    if (spdsConfig == NULL) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("SyncManager configuration not found: %s"), context);
        goto finally;
    }
    
    /*
    * if sourceName is empty then is a property of root context
    */
    if (wcscmp(sourceName, TEXT("")) == 0) {
        
        if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_REMOTE_SETTINGS) == 0)
            spdsConfig->getPropertyValue(PROPERTY_SOURCEPROP_REMOTE_SETTINGS, buf, VALUESIZE); 
        else if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_SYSTEMTIME_TO_LOCALTIME) == 0)
            spdsConfig->getPropertyValue(PROPERTY_SOURCEPROP_SYSTEMTIME_TO_LOCALTIME, buf, VALUESIZE);         
        else
            wsprintf(buf, TEXT(""));
              
        wcscpy(propertyValue, buf);                                            
        
        return 0;
    }

    nc = spdsConfig->getChildrenCount();
    
    if (nc > 0) {
                lastErrorCode = 0;
        ManagementNode** children = new ManagementNode*[nc];

        spdsConfig->getChildren(children, &nc);
        if (lastErrorCode) {
                goto finally;
        }
        
        for (int i = 0; i < nc; ++i) {
                        
            if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {
                wsprintf(s, TEXT("%s"), APPOINTMENTS);
                
            }
            else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
                wsprintf(s, TEXT("%s"), CONTACTS);
            }
            else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
                wsprintf(s, TEXT("%s"), TASKS);
            }
            else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
                wsprintf(s, TEXT("%s"), NOTES);
            }
                        
            children[i]->getPropertyValue(PROPERTY_SOURCE_NAME, buf, VALUESIZE); 
            
            if (wcscmp(s, buf) == 0) {
                
                if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_SHOW) == 0) {
                     
                    children[i]->getPropertyValue(PROPERTY_SOURCEPROP_SHOW, buf, VALUESIZE); 
                }
                else {
                    wsprintf(buf, TEXT(""));                   
                }
                
                wcscpy(propertyValue, buf); 

                delete children[i];
                
                break;
            }
            
            delete children[i];

        }
       
        delete [] children;
    }
    
finally:
    
    return 0;
    
}