/*
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include <windows.h>
#include <initguid.h>
#include <ole2.h>
#include <mstask.h>
#include <msterr.h>
#include <objidl.h>
#include <wchar.h>
#include <stdio.h>
#include <errno.h>
#include "spdm/common/DeviceManager.h"
#include "spdm/common/DeviceManagerFactory.h"
#include "outlook/winmaincpp.h"

#define COMMENT             TEXT("Sync4j SyncClient Outlook scheduler")
#define APP_NAME            TEXT("Sync4j SyncClient Outlook")
#define NEVER               TEXT("Never")
#define EVERY_DAY           TEXT("Every day")
#define PARAM_SCHED         TEXT("schedule")
#define ACCOUNT_NAME        TEXT("accountName"    )
#define ACCOUNT_PASSWORD    TEXT("accountPassword")
#define INSTALL_DIR         TEXT("installDir")
#define PROGRAM_NAME        TEXT("SyncClientOutlook.exe")


#define ACCOUNT_CREDENTIAL_ERROR    TEXT("The last task excecution was not performed. Please check account information")
#define ACCOUNT_CREDENTIAL_ERROR    TEXT("The last task excecution was not performed. Please check account information")
#define GENERIC_ERROR               TEXT("The last task excecution was not performed. Please check windows task scheduler for more details")
#define OK_ERROR                    TEXT("The last task excecution was performed successfully")
int getIntFromChar(LPCSTR string) {
    
    int l = 0, ret = 0;
    wchar_t* wstring = NULL;
    
    l = strlen(string);
    wstring = new wchar_t[l + 1];
    MultiByteToWideChar (CP_ACP, 0, string, l, wstring, l);
    wstring[l] = 0;
    
    errno = 0;
    ret = wcstol(wstring, NULL, 0);
    
    //
    // an error occurs
    //
    if (errno != 0) {
        ret = -1;
    }
    delete wstring;
    return ret;
}

wchar_t* getWcharFromChar(LPCSTR string) {
    
    int l = 0, ret = 0;
    wchar_t* wstring = NULL;
    
    l = strlen(string);
    wstring = new wchar_t[l + 1];
    MultiByteToWideChar (CP_ACP, 0, string, l, wstring, l);
    wstring[l] = 0;
   
    return wstring;
}

void setCharFieldFromWchar(char* field, wchar_t* wfield, unsigned int size) {
    
    BOOL usedDefaultChar;
    char internalBuffer[512];
    memset(internalBuffer, 511, 0);
    
    WideCharToMultiByte (CP_ACP, 0, wfield, 512 -1, internalBuffer, 512 -1, "?", &usedDefaultChar);     

    if (size > strlen(internalBuffer) ) {
        strcpy(field, internalBuffer);  
        field[strlen(field)] = ' ';
    }
    
}

/*
* Get the propertyValue of the property accountName and accountPassword used to log in
* to start the scheduled task properly and the installation directory
*/

void getProperty(wchar_t* propertyName, wchar_t* propertyValue, int size) {

    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();
    ManagementNode* spdsConfig = dm->getManagementNode(APPLICATION_URI);
    spdsConfig->getPropertyValue(propertyName, propertyValue, size);          
    
    if (dm != NULL) {
		delete dm;
	}

	if (spdsConfig != NULL) {
		delete spdsConfig;
	}

}

void setProperty(wchar_t* propertyName, wchar_t* propertyValue) {

    DeviceManagerFactory factory = DeviceManagerFactory();
    DeviceManager* dm = factory.getDeviceManager();
    ManagementNode* spdsConfig = dm->getManagementNode(APPLICATION_URI);
    spdsConfig->setPropertyValue(propertyName, propertyValue);     
    
    if (dm != NULL) {
		delete dm;
	}

	if (spdsConfig != NULL) {
		delete spdsConfig;
	}

}


/*
* Create a new Task into task scheduler. It takes values to insert from VB Schedule form.
* 
* Freq - NEVER or DAILY
* Start our - hh starting (00 - 24)
* Start minute - mm starting (00 - 59)
* StartDay
* StartMonth
* StartYear
* DayNum - repeate the task every <DayNum>
* MinNum - repeate the task every <MinNum>
*
*/
DWORD WINAPI createTask (LPCSTR Frequency  , 
                         LPCSTR StartHour  , 
                         LPCSTR StartMinute,
                         LPCSTR StartDay   ,
                         LPCSTR StartMonth ,
                         LPCSTR StartYear  ,
                         LPCSTR DayNum     ,
                         LPCSTR MinNum     ,
                         LPCSTR AccountName,
                         LPCSTR AccountPassword) {

  HRESULT hr = S_OK;
  ITaskScheduler *pITS;
  
  wchar_t *frequency       = NULL;
  wchar_t *accountName     = NULL;
  wchar_t *accountPassword = NULL;
  int startHour   = 0,
      startMinute = 0,
      startDay    = 0,
      startMonth  = 0,
      startYear   = 0,
      dayNum      = 0,
      minNum      = 0;

  
  ////////////////////////////////////////////////////////////////
  // Initialize every variable
  ////////////////////////////////////////////////////////////////
  wchar_t temp[256];
  frequency         = getWcharFromChar(Frequency        );
  frequency         = getWcharFromChar(Frequency        );
  startHour         = getIntFromChar(StartHour          );
  startMinute       = getIntFromChar(StartMinute        );
  startDay          = getIntFromChar(StartDay           );
  startMonth        = getIntFromChar(StartMonth         );
  startYear         = getIntFromChar(StartYear          );  
  dayNum            = getIntFromChar(DayNum             );
  minNum            = getIntFromChar(MinNum             );
  wsprintf(temp, TEXT("%i"), minNum);
  /////////////////////////////////////////////////////////////////
  // Call CoInitialize to initialize the COM library and then 
  // CoCreateInstance to get the Task Scheduler object. 
  /////////////////////////////////////////////////////////////////
  hr = CoInitialize(NULL);
  if (SUCCEEDED(hr))
  {
     hr = CoCreateInstance(CLSID_CTaskScheduler,
                           NULL,
                           CLSCTX_INPROC_SERVER,
                           IID_ITaskScheduler,
                           (void **) &pITS);
     if (FAILED(hr))
     {
        CoUninitialize();
        return 1;
     }
  }
  else
  {
     return 1;
  }
  
  
  /////////////////////////////////////////////////////////////////
  // Call ITaskScheduler::NewWorkItem to create new task.
  /////////////////////////////////////////////////////////////////
  LPCWSTR pwszTaskName;
  
  ITask *pITask;
  IPersistFile *pIPersistFile;
  pwszTaskName = APP_NAME;
  
  wchar_t t[512];
  wchar_t path[512];
  
  getProperty(INSTALL_DIR, t, 512);
  
  wsprintf(path, TEXT("\"%s\\%s\""),t, PROGRAM_NAME);
 
  hr = pITS->NewWorkItem(pwszTaskName,           // Name of task
                         CLSID_CTask,            // Class identifier 
                         IID_ITask,              // Interface identifier
                         (IUnknown**)&pITask);   // Address of task interface
  
  
  pITS->Release();                               // Release object
  if (FAILED(hr))
  {
     CoUninitialize();
     fprintf(stderr, "Failed calling NewWorkItem, error = 0x%x\n",hr);
     return 1;
  }
  
  //
  // set starting information about the new task
  //
  pITask->SetApplicationName (path);
  pITask->SetComment(COMMENT);
 
  
  if (wcscmp(NEVER, frequency) == 0) {
    pITask->SetFlags(TASK_FLAG_DISABLED);
  } else {
     pITask->SetFlags(TASK_FLAG_INTERACTIVE);
    //
    // Set information about Account user and store it into DM
    //
    accountName       = getWcharFromChar(AccountName      );
    accountPassword   = getWcharFromChar(AccountPassword  );
    setProperty(ACCOUNT_NAME,     accountName    );
    setProperty(ACCOUNT_PASSWORD, accountPassword);
    pITask->SetAccountInformation (accountName, accountPassword);

  }
  
  //
  // set parameter to launch sync in background
  // 
  pITask->SetParameters (PARAM_SCHED);

  
  
  //
  // Set information about the scheduling task
  //
  ///////////////////////////////////////////////////////////////////
  // Call ITask::CreateTrigger to create new trigger.
  ///////////////////////////////////////////////////////////////////
  
  ITaskTrigger *pITaskTrigger;
  WORD piNewTrigger;
  hr = pITask->CreateTrigger(&piNewTrigger,
                             &pITaskTrigger);
  if (FAILED(hr))
  {
    wprintf(L"Failed calling ITask::CreatTrigger: ");
    wprintf(L"error = 0x%x\n",hr);
    CoUninitialize();
    return 1;
  }
  
  
  //////////////////////////////////////////////////////
  // Define TASK_TRIGGER structure. Note that wBeginDay,
  // wBeginMonth, and wBeginYear must be set to a valid 
  // day, month, and year respectively.
  //////////////////////////////////////////////////////
  
  TASK_TRIGGER pTrigger;
  ZeroMemory(&pTrigger, sizeof (TASK_TRIGGER));
  
  // Add code to set trigger structure?
  pTrigger.wBeginDay                = startDay   ;      // Required
  pTrigger.wBeginMonth              = startMonth ;      // Required
  pTrigger.wBeginYear               = startYear  ;      // Required
  pTrigger.cbTriggerSize = sizeof (TASK_TRIGGER) ; 
  pTrigger.wStartHour               = startHour  ;
  pTrigger.wStartMinute             = startMinute;
  
  //
  // if minNum = -1 then the checkbox repeate task was unchecked 
  //

  if (minNum > -1) { 
      pTrigger.MinutesDuration          = 1440;  // a day
      pTrigger.MinutesInterval          = minNum;
  }
  pTrigger.TriggerType = TASK_TIME_TRIGGER_DAILY;
  pTrigger.Type.Daily.DaysInterval  = dayNum;
  
  
  ///////////////////////////////////////////////////////////////////
  // Call ITaskTrigger::SetTrigger to set trigger criteria.
  ///////////////////////////////////////////////////////////////////
  
  hr = pITaskTrigger->SetTrigger (&pTrigger);
  if (FAILED(hr))
  {
    wprintf(L"Failed calling ITaskTrigger::SetTrigger: ");
    wprintf(L"error = 0x%x\n",hr);
    CoUninitialize();
    return 1;
  }


  
  /////////////////////////////////////////////////////////////////
  // Call IUnknown::QueryInterface to get a pointer to 
  // IPersistFile and IPersistFile::Save to save 
  // the new task to disk.
  /////////////////////////////////////////////////////////////////
  
  hr = pITask->QueryInterface(IID_IPersistFile,
                              (void **)&pIPersistFile);
  
  pITask->Release();
  if (FAILED(hr))
  {
     CoUninitialize();
     fprintf(stderr, "Failed calling QueryInterface, error = 0x%x\n",hr);
     return 1;
  }
  
  pITaskTrigger->Release();


  hr = pIPersistFile->Save(NULL,
                           TRUE);
  pIPersistFile->Release();
  if (FAILED(hr))
  {
     CoUninitialize();
     fprintf(stderr, "Failed calling Save, error = 0x%x\n",hr);
     return 1;
  }
  
  
  printf("Created task.\n");


  return 0;
}


DWORD WINAPI updateTask (LPCSTR Frequency  , 
                         LPCSTR StartHour  , 
                         LPCSTR StartMinute,
                         LPCSTR StartDay   ,
                         LPCSTR StartMonth ,
                         LPCSTR StartYear  ,
                         LPCSTR DayNum     ,
                         LPCSTR MinNum     ,
                         LPCSTR AccountName,     
                         LPCSTR AccountPassword) {

  HRESULT hr = S_OK;
  ITaskScheduler *pITS;
  
  wchar_t *frequency       = NULL;
  wchar_t *accountName     = NULL;
  wchar_t *accountPassword = NULL;
  int startHour   = 0,
      startMinute = 0,
      startDay    = 0,
      startMonth  = 0,
      startYear   = 0,
      dayNum      = 0,
      minNum      = 0;

  
  ////////////////////////////////////////////////////////////////
  // Initialize every variable
  ////////////////////////////////////////////////////////////////

  frequency   = getWcharFromChar(Frequency  );  
  startHour   = getIntFromChar(StartHour    );
  startMinute = getIntFromChar(StartMinute  );
  startDay    = getIntFromChar(StartDay     );
  startMonth  = getIntFromChar(StartMonth   );
  startYear   = getIntFromChar(StartYear    );  
  dayNum      = getIntFromChar(DayNum       );
  minNum      = getIntFromChar(MinNum       );

  /////////////////////////////////////////////////////////////////
  // Call CoInitialize to initialize the COM library and then 
  // CoCreateInstance to get the Task Scheduler object. 
  /////////////////////////////////////////////////////////////////
  hr = CoInitialize(NULL);
  if (SUCCEEDED(hr))
  {
     hr = CoCreateInstance(CLSID_CTaskScheduler,
                           NULL,
                           CLSCTX_INPROC_SERVER,
                           IID_ITaskScheduler,
                           (void **) &pITS);
     if (FAILED(hr))
     {
        CoUninitialize();
        return 1;
     }
  }
  else
  {
     return 1;
  }
  
  
  /////////////////////////////////////////////////////////////////
  // Call ITaskScheduler::Activate to create new task.
  /////////////////////////////////////////////////////////////////
  IPersistFile *pIPersistFile;
  ITask *pITask;
  LPCWSTR lpcwszTaskName;
  lpcwszTaskName = APP_NAME;
  hr = pITS->Activate(lpcwszTaskName,
                      IID_ITask,
                      (IUnknown**) &pITask);
  
  // Release the ITaskScheduler interface.
  pITS->Release();  
  
  if (FAILED(hr))
  {
    wprintf(L"Failed calling ITaskScheduler::Activate: ");
    wprintf(L"error = 0x%x\n",hr);
    CoUninitialize();
    return 1;
  }
  
  
  if (wcscmp(NEVER, frequency) == 0) {
    pITask->SetFlags(TASK_FLAG_DISABLED);
  } else {
    pITask->SetFlags(TASK_FLAG_INTERACTIVE);
    //
    // Set information about Account user and store it into DM
    //
    accountName       = getWcharFromChar(AccountName      );
    accountPassword   = getWcharFromChar(AccountPassword  );
    setProperty(ACCOUNT_NAME,     accountName    );
    setProperty(ACCOUNT_PASSWORD, accountPassword);
    pITask->SetAccountInformation (accountName, accountPassword);
  }
 

  //
  // Set information about the scheduling task
  //
  ///////////////////////////////////////////////////////////////////
  // Call ITask::CreateTrigger to create new trigger.
  ///////////////////////////////////////////////////////////////////
  
  ITaskTrigger *pITaskTrigger;
  
  hr = pITask->GetTrigger(0, &pITaskTrigger);
  if (FAILED(hr))
  {
    wprintf(L"Failed calling ITask::getTrigger: ");
    wprintf(L"error = 0x%x\n",hr);
    CoUninitialize();
    return 1;
  }
    
  //////////////////////////////////////////////////////
  // Define TASK_TRIGGER structure. Note that wBeginDay,
  // wBeginMonth, and wBeginYear must be set to a valid 
  // day, month, and year respectively.
  //////////////////////////////////////////////////////
  
  TASK_TRIGGER pTrigger;
  ZeroMemory(&pTrigger, sizeof (TASK_TRIGGER));
  pITaskTrigger->GetTrigger(&pTrigger);
  // Add code to set trigger structure?
  pTrigger.wBeginDay                = startDay   ;      // Required
  pTrigger.wBeginMonth              = startMonth ;      // Required
  pTrigger.wBeginYear               = startYear  ;      // Required
  pTrigger.cbTriggerSize = sizeof (TASK_TRIGGER) ; 
  pTrigger.wStartHour               = startHour  ;
  pTrigger.wStartMinute             = startMinute;
  
  //
  // if minNum = -1 then the checkbox repeate task was unchecked 
  //

  if (minNum > -1) { 
      pTrigger.MinutesDuration          = 1440;  // a day
      pTrigger.MinutesInterval          = minNum;
  } else {
      pTrigger.MinutesDuration          = 0;
      pTrigger.MinutesInterval          = 0;
  }
  pTrigger.TriggerType = TASK_TIME_TRIGGER_DAILY;
  pTrigger.Type.Daily.DaysInterval  = dayNum;
  
  
  ///////////////////////////////////////////////////////////////////
  // Call ITaskTrigger::SetTrigger to set trigger criteria.
  ///////////////////////////////////////////////////////////////////
  
  hr = pITaskTrigger->SetTrigger (&pTrigger);
  if (FAILED(hr))
  {
    wprintf(L"Failed calling ITaskTrigger::SetTrigger: ");
    wprintf(L"error = 0x%x\n",hr);
    CoUninitialize();
    return 1;
  }


  
  /////////////////////////////////////////////////////////////////
  // Call IUnknown::QueryInterface to get a pointer to 
  // IPersistFile and IPersistFile::Save to save 
  // the new task to disk.
  /////////////////////////////////////////////////////////////////
  
  hr = pITask->QueryInterface(IID_IPersistFile,
                              (void **)&pIPersistFile);
  
  pITask->Release();
  if (FAILED(hr))
  {
     CoUninitialize();
     fprintf(stderr, "Failed calling QueryInterface, error = 0x%x\n",hr);
     return 1;
  }
  
  pITaskTrigger->Release();


  hr = pIPersistFile->Save(NULL,
                           TRUE);
  pIPersistFile->Release();
  if (FAILED(hr))
  {
     CoUninitialize();
     fprintf(stderr, "Failed calling Save, error = 0x%x\n",hr);
     return 1;
  }
  
  
  printf("Created task.\n");
  return 0;
}




/*
*
*/
DWORD WINAPI getTask(LPCSTR Frequency  , 
                     LPCSTR StartHour  , 
                     LPCSTR StartMinute,
                     LPCSTR StartDay   ,
                     LPCSTR StartMonth ,
                     LPCSTR StartYear  ,
                     LPCSTR DayNum     ,
                     LPCSTR MinNum     ,
                     LPCSTR AccountName,
                     LPCSTR AccountPassword,
                     LPCSTR Status,
                     unsigned int size) {

  HRESULT hr = S_OK;
  DWORD ret = 0;
  wchar_t temp[512];

  ///////////////////////////////////////////////////////////////////
  // Call CoInitialize to initialize the COM library and then
  // CoCreateInstance to get the Task Scheduler object.
  ///////////////////////////////////////////////////////////////////
  ITaskScheduler *pITS;
  
  hr = CoInitialize(NULL);
  if (SUCCEEDED(hr))
  {
    hr = CoCreateInstance(CLSID_CTaskScheduler,
                          NULL,
                          CLSCTX_INPROC_SERVER,
                          IID_ITaskScheduler,
                          (void **) &pITS);
    if (FAILED(hr)) {
      CoUninitialize();
      return 1;
    }
  }
  else
  {
    return 1;
  }  
  
  ///////////////////////////////////////////////////////////////////
  // Call ITaskScheduler::Activate to get the Task object.
  ///////////////////////////////////////////////////////////////////
  
  ITask *pITask;
  LPCWSTR lpcwszTaskName;
  lpcwszTaskName = APP_NAME;
  hr = pITS->Activate(lpcwszTaskName,
                      IID_ITask,
                      (IUnknown**) &pITask);
  
  // Release the ITaskScheduler interface.
  pITS->Release();  
  
  if (FAILED(hr))
  {
    wprintf(L"Failed calling ITaskScheduler::Activate: ");
    wprintf(L"error = 0x%x\n",hr);
    CoUninitialize();
    return 1;
  }
  
  DWORD pdwExitCode;
  HRESULT  hres = pITask->GetExitCode(&pdwExitCode);  
  if ((hres & SCHED_S_TASK_HAS_NOT_RUN) != 0) {
    setCharFieldFromWchar((char*)Status, GENERIC_ERROR, size);    
  }
  else if (hres != S_OK) { // generic error
    setCharFieldFromWchar((char*)Status, GENERIC_ERROR, size);
  } else {  // status OK!
    setCharFieldFromWchar((char*)Status, OK_ERROR, size);
  }
  

  //
  // get the flag to set NEVER or EVERY_DAY
  //
  DWORD flag = -1, triggerType = -1;
  pITask->GetFlags(&flag);
  
  if ((flag & TASK_FLAG_DISABLED) != 0) {    
    setCharFieldFromWchar((char*)Frequency, NEVER, size);
  } else {
    setCharFieldFromWchar((char*)Frequency, EVERY_DAY, size);
  } 

  ITaskTrigger *pITaskTrigger;
  
  hr = pITask->GetTrigger(0, &pITaskTrigger);
  if (FAILED(hr))
  {
    wprintf(L"Failed calling ITask::getTrigger: ");
    wprintf(L"error = 0x%x\n",hr);
    CoUninitialize();
    return 1;
  }
  getProperty(ACCOUNT_NAME,     temp,     512);
  setCharFieldFromWchar((char*)AccountName, temp, size);
  getProperty(ACCOUNT_PASSWORD, temp,     512);
  setCharFieldFromWchar((char*)AccountPassword, temp, size);
  
  TASK_TRIGGER pTrigger;
  ZeroMemory(&pTrigger, sizeof (TASK_TRIGGER));
   
  pITaskTrigger->GetTrigger(&pTrigger);
  
  wsprintf(temp, TEXT("%i"), pTrigger.wStartHour);
  setCharFieldFromWchar((char*)StartHour, temp, size);
  
  wsprintf(temp, TEXT("%i"), pTrigger.wStartMinute);
  setCharFieldFromWchar((char*)StartMinute, temp, size);
  
  wsprintf(temp, TEXT("%i"), pTrigger.wBeginDay );
  setCharFieldFromWchar((char*)StartDay, temp, size);
  
  wsprintf(temp, TEXT("%i"), pTrigger.wBeginMonth );
  setCharFieldFromWchar((char*)StartMonth, temp, size);
  
  wsprintf(temp, TEXT("%i"), pTrigger.wBeginYear );
  setCharFieldFromWchar((char*)StartYear, temp, size);
  
  //wsprintf(temp, TEXT("%i"), pTrigger.MinutesDuration);

  
  wsprintf(temp, TEXT("%i"), pTrigger.MinutesInterval);
  setCharFieldFromWchar((char*)MinNum, temp, size);
  
  
  wsprintf(temp, TEXT("%i"), pTrigger.Type.Daily.DaysInterval);
  setCharFieldFromWchar((char*)DayNum, temp, size);
  
  //
  // Check if minuteDuration != 1440 is set manually. Do we won't handle it
  //
  if (pTrigger.MinutesDuration != 1440) {
    setCharFieldFromWchar((char*)MinNum, TEXT("-1"), size);

  }

  triggerType = pTrigger.TriggerType;
  wsprintf(temp, TEXT("%i"), triggerType);

  if (triggerType != TASK_TIME_TRIGGER_DAILY || 
                    (pTrigger.MinutesDuration != 0 
                    && pTrigger.MinutesInterval != 0 
                    && pTrigger.MinutesDuration != 1440)
            ) {    
    return 2;
  }
  
  // Release the ITask interface.
  pITaskTrigger->Release();
  pITask->Release();  
  
  CoUninitialize();
  return 0;
}

/*
*
*/

DWORD WINAPI deleteTask() {

  HRESULT hr = S_OK;

  ///////////////////////////////////////////////////////////////////
  // Call CoInitialize to initialize the COM library and then
  // CoCreateInstance to get the Task Scheduler object.
  ///////////////////////////////////////////////////////////////////
  ITaskScheduler *pITS;
  
  hr = CoInitialize(NULL);
  if (SUCCEEDED(hr))
  {
    hr = CoCreateInstance(CLSID_CTaskScheduler,
                          NULL,
                          CLSCTX_INPROC_SERVER,
                          IID_ITaskScheduler,
                          (void **) &pITS);
    if (FAILED(hr)) {
      CoUninitialize();
      return 1;
    }
  }
  else
  {
    return 1;
  }  
  
  ///////////////////////////////////////////////////////////////////
  // Call ITaskScheduler::Activate to get the Task object.
  ///////////////////////////////////////////////////////////////////

  LPCWSTR lpcwszTaskName;
  lpcwszTaskName = APP_NAME;
  hr = pITS->Delete(lpcwszTaskName);  
    
  pITS->Release();  
  
  return 0 ;
}
